import { Fragment, Host, h } from "@stencil/core";
import clsx from "clsx";
import { TextInput } from "./inputs/text";
import { fetchForm, saveResponse } from "../../utils/api";
import { ContentInput } from "./inputs/content";
import { TextareaInput } from "./inputs/textarea";
import { createGetClass } from "../../utils/classes";
import { Fetcher } from "../fetcher/fetcher";
import { Success } from "./common/success";
import { Content } from "./common/content";
export class HappyreactForm {
  constructor() {
    this.classes = undefined;
    this.mode = 'flow';
    this.state = {
      valid: false,
      dirty: false,
      values: {},
      errors: {
        _count: 0
      }
    };
    this.response = {
      status: 'loading',
      data: null
    };
    this.request = {
      status: 'idle',
      data: null
    };
    this.isVisible = false;
    this.props = null;
  }
  handleSend() {
    this.request.status = 'loading';
    saveResponse({
      token: this.props.token,
      reactionId: this.props.reactionId,
      url: this.props.url,
      values: this.state.values,
      resources: this.props.resources,
      count: this.props.count
    })
      .then((response) => {
      this.request = {
        status: 'success',
        data: response
      };
    });
  }
  handleValidate(value, field, state) {
    for (const validation of field.validations) {
      switch (validation.type) {
        case 'REQUIRED':
          if (!value) {
            const message = `${field.label} is required`;
            state.errors[field.id] = message;
            throw new Error(message);
          }
        default:
          break;
      }
    }
    this.state = state;
  }
  _countErrors(state) {
    return Object.keys(state.errors).filter((key) => {
      if (key.includes('_')) {
        return false;
      }
      return !!state.errors[key];
    }).length;
  }
  handleSubmit(event) {
    event.preventDefault();
    const state = structuredClone(this.state);
    for (const field of this.response.data.fields) {
      try {
        this.handleValidate(state.values[field.id], field, state);
      }
      catch (cause) {
        state.errors._count = this._countErrors(state);
      }
      finally {
        this.state = state;
      }
    }
    if (state.errors._count === 0) {
      this.handleSend();
    }
  }
  handleInput(event, field) {
    //@ts-expect-error
    const value = event.target.value;
    const state = structuredClone(this.state);
    state.dirty = true;
    state.values[field.id] = value;
    try {
      this.handleValidate(value, field, state);
      state.errors[field.id] = null;
    }
    catch (cause) {
      state.errors[field.id] = cause.message;
    }
    state.errors._count = this._countErrors(state);
    this.state = state;
  }
  /**
   * onReaction event handler
   * @param reaction
   */
  onCloseHandler() {
    this.close.emit();
    this.isVisible = false;
  }
  showHandler(event) {
    this.props = event.detail;
    this.state = {
      valid: false,
      dirty: false,
      values: {},
      errors: {
        _count: 0
      }
    };
    if (this.response.status === 'idle') {
      this.response = {
        status: 'loading',
        data: null
      };
    }
    fetchForm({
      token: this.props.token,
      reactionId: this.props.reactionId,
      url: this.props.url
    })
      .then((response) => {
      this.response = {
        status: 'success',
        data: response
      };
      this.request = {
        status: 'idle',
        data: null
      };
      this.isVisible = true;
    })
      .catch((cause) => {
      if (cause.message.includes('Cannot find form for reaction')) {
        this.response = {
          status: 'idle',
          data: null
        };
        return;
      }
      this.response = {
        status: 'error',
        data: cause.message
      };
    });
  }
  handleClick(event) {
    if (this.host.isSameNode(event.target)) {
      if (this.mode === 'modal') {
        this.onCloseHandler();
      }
    }
  }
  _getInput(field, getClass) {
    switch (field.type) {
      case 'TEXT':
        return h(TextInput, { id: field.id, key: field.id, onInput: (event) => this.handleInput(event, field), name: field.id, label: field.label, hint: field.hint, error: this.state.errors[field.id], getClass: getClass });
      case 'TEXTAREA':
        return h(TextareaInput, { key: field.id, onInput: (event) => this.handleInput(event, field), name: field.id, label: field.label, hint: field.hint, error: this.state.errors[field.id], getClass: getClass });
      case 'CONTENT':
        return h(ContentInput, { key: field.id, onInput: (event) => this.handleInput(event, field), name: field.id, label: field.label, hint: field.hint, error: this.state.errors[field.id] });
      default:
        throw new Error('Cannot match field type');
    }
  }
  render() {
    const getClass = createGetClass(this.classes);
    return this.isVisible ? (h(Host, { class: clsx('root', getClass('root')), onClick: this.handleClick.bind(this) }, h("div", { class: clsx("hr-form-container", this.isVisible && "hr-form-container-visible", getClass('container'), getClass('containerVisible')) }, h(Fetcher, { status: this.response.status }, this.request.status === 'success' ? (h(Success, { icon: this.request.data.icon, content: this.request.data.content, getClass: getClass })) : this.response.status === 'success' && (h(Fragment, null, this.mode === 'modal' && (h("div", { class: clsx('hr-close hr-close-top', getClass('close'), getClass('closeTop')) }, h("button", { onClick: this.onCloseHandler.bind(this), type: "button", class: clsx('hr-close-top-button', getClass('closeTopButton')) }, "X"))), this.response.data.content.before && (h(Content, { content: this.response.data.content.before, classes: clsx(getClass('content'), getClass('contentBefore')) })), h("form", { class: clsx("hr-form", getClass('form')), onSubmit: this.handleSubmit.bind(this) }, this.state.errors._count > 0 && (h("output", { class: clsx("hr-form-header", getClass('formHeader')), htmlFor: this.response.data.fields.map(field => field.id).join(' '), name: "errors" }, this.state.errors._count, " errors found:", h("ul", { class: clsx("hr-form-errors-list", getClass('formErrorsList')) }, this.response.data.fields.map((field) => this.state.errors[field.id] ? (h("li", { class: clsx("hr-form-error-item", getClass('formErrorItem')) }, this.state.errors[field.id])) : null)))), this.response.data.fields.map((field) => {
      return this._getInput(field, getClass);
    }), h("div", { class: clsx("hr-footer", getClass('footer')) }, this.request.status === 'idle' && (h("button", { type: "submit", class: clsx("hr-submit hr-submit-idle", getClass('submit'), getClass('submitIdle')) }, "Send")), this.request.status === 'loading' && (h("button", { type: "submit", class: clsx("hr-submit hr-submit-loading", getClass('submit'), getClass('submitIdle')), disabled: true }, "Saving...")))), this.response.data.content.after && (h(Content, { content: this.response.data.content.after, classes: clsx(getClass('content'), getClass('contentAfter')) })), this.mode === 'modal' && (h("div", { class: clsx('hr-close hr-close-bottom', getClass('close'), getClass('closeBottom')) }, h("button", { onClick: this.onCloseHandler.bind(this), type: "button", class: clsx('hr-close-bottom-button', getClass('closeBottomButton')) }, "Close"))))))))) : null;
  }
  static get is() { return "hr-form"; }
  static get originalStyleUrls() {
    return {
      "$": ["hr-form.css"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["hr-form.css"]
    };
  }
  static get properties() {
    return {
      "classes": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "Readonly<{ [key in keyof FormClasses]: FormClasses[key] }>",
          "resolved": "{ readonly root?: string; readonly container?: string; readonly containerVisible?: string; readonly success?: string; readonly successIcon?: string; readonly successContent?: string; readonly content?: string; readonly contentBefore?: string; readonly contentAfter?: string; readonly form?: string; readonly formHeader?: string; readonly formErrorsList?: string; readonly formErrorItem?: string; readonly footer?: string; readonly submit?: string; readonly submitIdle?: string; readonly submitLoading?: string; readonly field?: string; readonly label?: string; readonly labelText?: string; readonly input?: string; readonly inputHint?: string; readonly inputError?: string; readonly inputText?: string; readonly close?: string; readonly closeTop?: string; readonly closeBottom?: string; readonly closeTopButton?: string; readonly closeBottomButton?: string; }",
          "references": {
            "Readonly": {
              "location": "global",
              "id": "global::Readonly"
            },
            "FormClasses": {
              "location": "import",
              "path": "../../types/props",
              "id": "src/types/props.ts::FormClasses"
            },
            "key": {
              "location": "global",
              "id": "global::key"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Classes"
        }
      },
      "mode": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "'flow' | 'modal'",
          "resolved": "\"flow\" | \"modal\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "modes"
        },
        "attribute": "mode",
        "reflect": false,
        "defaultValue": "'flow'"
      }
    };
  }
  static get states() {
    return {
      "state": {},
      "response": {},
      "request": {},
      "isVisible": {},
      "props": {}
    };
  }
  static get events() {
    return [{
        "method": "close",
        "name": "close",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Event fired when user click reaction"
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "host"; }
  static get listeners() {
    return [{
        "name": "show",
        "method": "showHandler",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
//# sourceMappingURL=hr-form.js.map
