import { Host, h } from "@stencil/core";
import { DEFAULT_URL } from "../../utils/constants";
import { collectReaction, fetchProject } from "../../utils/api";
import clsx from "clsx";
import { createGetClass } from "../../utils/classes";
import { formatTemplate } from "../../utils/strings";
import { formatShortNumber } from "../../utils/numbers";
import { Fetcher } from "../fetcher/fetcher";
export class HrWidget {
  constructor() {
    /**
     * Form element
     */
    this.form = document.querySelector('hr-form');
    this.token = undefined;
    this.resource = undefined;
    this.classes = undefined;
    this.url = DEFAULT_URL;
    this.response = {
      status: 'loading',
      data: null
    };
  }
  _isFirstReaction() {
    var _a;
    return !((_a = this.response.data.reacted) === null || _a === void 0 ? void 0 : _a.id);
  }
  _isUnlimitedReactions() {
    return this.response.data.preferences.allowMultiple && this.response.data.preferences.maxReactions === 0;
  }
  _isLimitReactionsNotReached(reaction) {
    return this.response.data.reacted.id === reaction.id &&
      this.response.data.preferences.allowMultiple &&
      this.response.data.reacted.count < this.response.data.preferences.maxReactions;
  }
  _isLimitReactionsReached() {
    return this.response.data.reacted.count >= this.response.data.preferences.maxReactions;
  }
  _getAffiliateURL() {
    if (typeof window !== 'undefined' && 'location' in window) {
      return `https://happyreact.com/?utm_source=${window.location.origin}&utm_medium=widget&utm_campaign=footer`;
    }
    return `https://happyreact.com/?utm_source=feedback&utm_medium=widget&utm_campaign=footer`;
  }
  async collect(reaction) {
    if (this.response.status !== 'success') {
      return;
    }
    const data = structuredClone(this.response.data);
    if (this._isFirstReaction() ||
      this._isUnlimitedReactions() ||
      this._isLimitReactionsNotReached(reaction)) {
      const foundReaction = data.reactions.find((currentReaction) => currentReaction.id === reaction.id);
      if (!data.reacted.id) {
        data.reacted.id = reaction.id;
        data.reacted.count = 1;
      }
      else {
        data.reacted.count += 1;
      }
      foundReaction.count += 1;
      this.response = {
        status: 'success',
        data
      };
      const response = await collectReaction({
        token: this.token,
        resource: this.resource,
        reactionId: reaction.id,
        url: this.url,
        count: 1
      });
      this.onReactionHandler(reaction, { resources: response.resources });
    }
    else if (this._isLimitReactionsReached()) {
      this.onReactionsLimitReachedHandler();
    }
  }
  /**
   * onReaction event handler
   * @param reaction
   */
  onReactionHandler(reaction, data) {
    this.reaction.emit(reaction);
    if (this.form) {
      this.form.dispatchEvent(new CustomEvent('show', {
        detail: {
          token: this.token,
          reactionId: reaction.id,
          url: this.url,
          resources: data.resources,
          count: reaction.count
        }
      }));
    }
  }
  /**
   * onReactionsLimitReached event handler
   */
  onReactionsLimitReachedHandler() {
    this.reactionsLimitReached.emit();
  }
  componentWillLoad() {
    fetchProject({
      token: this.token,
      resource: this.resource,
      url: this.url
    })
      .then((response) => {
      this.response = {
        status: 'success',
        data: response
      };
    })
      .catch((cause) => {
      this.response = {
        status: 'error',
        data: cause.message
      };
    });
  }
  render() {
    var _a;
    const getClass = createGetClass(this.classes);
    return (h(Host, { class: getClass('root') }, h("div", { class: clsx('hr-container', getClass('container')) }, h(Fetcher, { status: this.response.status }, h("div", { class: clsx('hr-grid', getClass('grid')) }, (_a = this.response.data) === null || _a === void 0 ? void 0 : _a.reactions.map((reaction) => (h("div", { class: clsx('hr-cell', getClass('cell')), key: reaction.id }, h("button", { onClick: () => this.collect(reaction), class: clsx('hr-reaction', getClass('reaction'), {
        [clsx('hr-reaction-active', getClass('reactionActive'))]: this.response.data.reacted.id === reaction.id
      }) }, h("span", { class: clsx('hr-reaction-icon', getClass('reactionIcon')) }, reaction.icon), reaction.label && (h("span", { class: clsx('hr-reaction-text', getClass('reactionText')) }, formatTemplate({
      template: reaction.label,
      data: {
        count: this.response.data.preferences.shortNumber
          ? String(formatShortNumber(reaction.count))
          : String(reaction.count)
      }
    })))))))), h("a", { href: this._getAffiliateURL(), rel: "noopener", target: "_blank", class: "hr-powered-by", ref: (node) => {
        var _a;
        if (node) {
          node.style.setProperty('display', !((_a = this.response.data) === null || _a === void 0 ? void 0 : _a.preferences.whitelabel) ? 'block' : 'none', 'important');
        }
      } }, "Feedback widget by Happy React")))));
  }
  static get is() { return "hr-widget"; }
  static get originalStyleUrls() {
    return {
      "$": ["hr-widget.css"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["hr-widget.css"]
    };
  }
  static get properties() {
    return {
      "token": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": true,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Token of your project"
        },
        "attribute": "token",
        "reflect": false
      },
      "resource": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": true,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Unique identifier of resource"
        },
        "attribute": "resource",
        "reflect": false
      },
      "classes": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string | WidgetClasses",
          "resolved": "WidgetClasses | string",
          "references": {
            "WidgetClasses": {
              "location": "import",
              "path": "../../types/props",
              "id": "src/types/props.ts::WidgetClasses"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Classes applied to widget elements. You can find them on [Happy React styling guide](https://happyreact.com/docs/configuration/classes)"
        },
        "attribute": "classes",
        "reflect": false
      },
      "url": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Url where should widget get data. Used when you have own Happy React instance"
        },
        "attribute": "url",
        "reflect": false,
        "defaultValue": "DEFAULT_URL"
      }
    };
  }
  static get states() {
    return {
      "response": {}
    };
  }
  static get events() {
    return [{
        "method": "reaction",
        "name": "reaction",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Event fired when user click reaction"
        },
        "complexType": {
          "original": "Reaction",
          "resolved": "Reaction",
          "references": {
            "Reaction": {
              "location": "import",
              "path": "../../types/data",
              "id": "src/types/data.ts::Reaction"
            }
          }
        }
      }, {
        "method": "reactionsLimitReached",
        "name": "reactionsLimitReached",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Event fired when user reach reactions limit"
        },
        "complexType": {
          "original": "Reaction",
          "resolved": "Reaction",
          "references": {
            "Reaction": {
              "location": "import",
              "path": "../../types/data",
              "id": "src/types/data.ts::Reaction"
            }
          }
        }
      }];
  }
}
//# sourceMappingURL=hr-widget.js.map
